@@ -11,7 +11,7 @@ use std::fmt;
 use alpm::{Alpm, Db, Dep, Depend, Version};
 use alpm_utils::{AsTarg, DbListExt};
 use log::Level::Debug;
-use log::{debug, error, log_enabled};
+use log::{debug, log_enabled};
 use raur::{ArcPackage, Cache, Raur, SearchBy};
 
 bitflags! {
@@ -226,19 +226,23 @@ impl<'a, 'b, H: Raur + Sync> Resolver<'a, 'b, H> {
     /// Devel packages are never skipped when using NEEDED.
     ///
     /// By default, no packages are considered devel.
-    pub fn is_devel<F: Fn(&str) -> bool + 'static>(mut self, f: F) -> Self {
+    pub fn devel_pkgs<F: Fn(&str) -> bool + 'static>(mut self, f: F) -> Self {
         self.is_devel = Some(IsDevel::new(f));
         self
     }
 
-    /// Causes `aur/foo` as meaning from the AUR, instead of a repo named `aur`.
-    pub fn aur_namespace(&mut self) {
-        self.aur_namespace = Some("aur".to_string());
+    /// If enabled, causes `aur/foo` to mean from the AUR, instead of a repo named `aur`.
+    pub fn aur_namespace(mut self, enable: bool) -> Self {
+        if enable {
+            self.aur_namespace = Some("aur".to_string());
+        }
+        self
     }
 
-    /// Causes `<name>/foo` as meaning from the AUR, instead of a repo named `<name>`.
-    pub fn custom_aur_namespace(&mut self, name: String) {
-        self.aur_namespace = Some(name);
+    /// Causes `<name>/foo` to mean from the AUR, instead of a repo named `<name>`.
+    pub fn custom_aur_namespace(mut self, name: Option<String>) -> Self {
+        self.aur_namespace = name;
+        self
     }
 
     /// Getter for the aur cache
@@ -505,9 +509,16 @@ impl<'a, 'b, H: Raur + Sync> Resolver<'a, 'b, H> {
 
         debug!("Caching done, building tree");
 
-        for aur_pkg in aur_targets {
+        for &aur_pkg in &aur_targets {
             let dep = Depend::new(aur_pkg);
-            if self.assume_installed(&dep) {
+
+            if !is_target
+                && ((!self.flags.contains(Flags::LOCAL_REPO) && self.satisfied_local(&dep))
+                    || (self.flags.contains(Flags::LOCAL_REPO)
+                        && self.find_repo_satisfier_silent(dep.to_string()).is_some()
+                        && self.satisfied_local(&dep))
+                    || self.assume_installed(&dep))
+            {
                 continue;
             }
 
@@ -521,10 +532,6 @@ impl<'a, 'b, H: Raur + Sync> Resolver<'a, 'b, H> {
                 continue;
             };
 
-            if !is_target && self.alpm.localdb().pkgs().find_satisfier(aur_pkg).is_some() {
-                continue;
-            }
-
             if self.flags.contains(Flags::NEEDED) || !is_target {
                 let is_devel = self
                     .is_devel
@@ -546,7 +553,7 @@ impl<'a, 'b, H: Raur + Sync> Resolver<'a, 'b, H> {
 
             let is_make = make.contains(&aur_pkg);
             self.stack.push(aur_pkg.to_string());
-            self.resolve_aur_pkg_deps(&pkg, is_make)?;
+            self.resolve_aur_pkg_deps(&aur_targets, &pkg, is_make)?;
             self.stack.pop().unwrap();
 
             if self
@@ -637,7 +644,12 @@ impl<'a, 'b, H: Raur + Sync> Resolver<'a, 'b, H> {
         }
     }
 
-    fn resolve_aur_pkg_deps(&mut self, pkg: &ArcPackage, make: bool) -> Result<(), Error> {
+    fn resolve_aur_pkg_deps(
+        &mut self,
+        targs: &[&str],
+        pkg: &ArcPackage,
+        make: bool,
+    ) -> Result<(), Error> {
         if !self.flags.contains(Flags::NO_DEPS) {
             let check = if self.flags.contains(Flags::CHECK_DEPENDS) {
                 Some(&pkg.check_depends)
@@ -654,25 +666,33 @@ impl<'a, 'b, H: Raur + Sync> Resolver<'a, 'b, H> {
             for dep_str in depends {
                 let dep = Depend::new(dep_str.to_string());
 
-                if self.satisfied_build(&dep)
-                    || (!self.flags.contains(Flags::LOCAL_REPO) && self.satisfied_local(&dep))
-                    || (self.flags.contains(Flags::LOCAL_REPO)
-                        && self.find_repo_satisfier_silent(dep.to_string()).is_some()
-                        && self.satisfied_local(&dep))
-                    || self.satisfied_install(&dep)
-                    || self.resolved.contains(&dep.to_string())
-                    || self.assume_installed(&dep)
+                if self.satisfied_build(&dep) {
+                    continue;
+                }
+
+                let is_aur_targ = self.dep_is_aur_targ(targs, &dep);
+
+                if !is_aur_targ
+                    && ((!self.flags.contains(Flags::LOCAL_REPO) && self.satisfied_local(&dep))
+                        || (self.flags.contains(Flags::LOCAL_REPO)
+                            && self.find_repo_satisfier_silent(dep.to_string()).is_some()
+                            && self.satisfied_local(&dep))
+                        || self.satisfied_install(&dep)
+                        || self.resolved.contains(&dep.to_string())
+                        || self.assume_installed(&dep))
                 {
                     continue;
                 }
 
                 self.resolved.insert(dep.to_string());
 
-                if let Some(pkg) = self.find_repo_satisfier(dep.to_string()) {
-                    self.stack.push(pkg.name().to_string());
-                    self.resolve_repo_pkg(pkg, false, true)?;
-                    self.stack.pop().unwrap();
-                    continue;
+                if !is_aur_targ {
+                    if let Some(pkg) = self.find_repo_satisfier(dep.to_string()) {
+                        self.stack.push(pkg.name().to_string());
+                        self.resolve_repo_pkg(pkg, false, true)?;
+                        self.stack.pop().unwrap();
+                        continue;
+                    }
                 }
 
                 let sat_pkg = if let Some(pkg) = self.select_satisfier_aur_cache(&dep, false) {
@@ -695,7 +715,7 @@ impl<'a, 'b, H: Raur + Sync> Resolver<'a, 'b, H> {
                 };
 
                 self.stack.push(dep_str.to_string());
-                self.resolve_aur_pkg_deps(&sat_pkg, true)?;
+                self.resolve_aur_pkg_deps(targs, &sat_pkg, true)?;
                 self.stack.pop();
 
                 let p = AurPackage {
@@ -741,6 +761,7 @@ impl<'a, 'b, H: Raur + Sync> Resolver<'a, 'b, H> {
             }
         }
 
+        debug!("pushing to install: {}", pkg.name());
         self.actions.install.push(RepoPackage { pkg, make, target });
 
         Ok(())
@@ -823,7 +844,7 @@ impl<'a, 'b, H: Raur + Sync> Resolver<'a, 'b, H> {
                         .search_by(word, SearchBy::NameDesc)
                         .await
                         .unwrap_or_else(|e| {
-                            error!("provide search '{}' failed: {}", word, e);
+                            debug!("provide search '{}' failed: {}", word, e);
                             Vec::new()
                         })
                         .into_iter()
@@ -994,8 +1015,23 @@ impl<'a, 'b, H: Raur + Sync> Resolver<'a, 'b, H> {
         pkg
     }
 
+    fn dep_is_aur_targ(&self, targs: &[&str], dep: &Dep) -> bool {
+        if let Some(pkg) = self.find_satisfier_aur_cache(&dep) {
+            for &targ in targs {
+                if satisfies_aur_pkg(
+                    &Depend::new(targ),
+                    pkg,
+                    self.flags.contains(Flags::NO_DEP_VERSION),
+                ) {
+                    return true;
+                }
+            }
+        }
+
+        false
+    }
+
     fn push_build(&mut self, pkgbase: &str, pkg: AurPackage) {
-        debug!("pushing to build: {}", pkg.pkg.name);
         for base in &mut self.actions.build {
             if base.package_base() == pkgbase {
                 base.pkgs.push(pkg);
@@ -1003,6 +1039,7 @@ impl<'a, 'b, H: Raur + Sync> Resolver<'a, 'b, H> {
             }
         }
 
+        debug!("pushing to build: {}", pkg.pkg.name);
         self.actions.build.push(Base { pkgs: vec![pkg] });
     }
 
@@ -1128,8 +1165,9 @@ mod tests {
         let alpm = alpm();
         let mut cache = HashSet::new();
 
-        let mut handle =
-            Resolver::new(&alpm, &mut cache, &raur, flags).provider_callback(|_, pkgs| {
+        let handle = Resolver::new(&alpm, &mut cache, &raur, flags)
+            .aur_namespace(true)
+            .provider_callback(|_, pkgs| {
                 debug!("provider choice: {:?}", pkgs);
                 if let Some(i) = pkgs.iter().position(|pkg| *pkg == "yay-bin") {
                     i
@@ -1138,7 +1176,6 @@ mod tests {
                 }
             });
 
-        handle.aur_namespace();
         let actions = handle.resolve_targets(pkgs).await.unwrap();
 
         let mut build = actions
